/**
 * Copyright (c) 2009-2011 Zauber S.A. <http://www.zaubersoftware.com/>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ar.com.zauber.commons.mom;

import net.sf.staccatocommons.restrictions.check.NonNull;

/**
 * {@link MapObjectMapper}s are objects capable of converting a complete graph
 * of objects into an structure-like maps - SLM -, and back.
 *
 * SLM format is specified bellow:
 *
 * TODO define format
 *
 * TODO define simple type, composite type, composite collection type
 *
 *
 *
 * @author flbulgarelli
 */
public interface MapObjectMapper {

    String JAVA_TYPE_KEY = "__javaType";

    /**
     * Converts an SLM into an instance of the class whose name the is value of
     * {@link #JAVA_TYPE_KEY}. If this key is absent, this message fails
     * throwing an {@link IllegalArgumentException}
     *
     * @param <A>
     * @param map
     * @return the SLM converted to an instance of the target type, or {@code
     *         null}, if {@code map} is {@code null}
     * @throws IllegalArgumentException
     *             if the object can not be converted into an instance of the
     *             requested type, because of unmatching keys or uncoersable
     *             properties
     */
    Object unmap(Object value);

    /**
     * Converts an SLM into an instance of the class whose name the is value of
     * {@value #JAVA_TYPE_KEY}, if present, or the given {@code type},
     * otherwise.
     *
     * Conversion rules are the following:
     *
     * <ol>
     * <li></li>
     * <li>If the given {@code value} is a {@link java.util.Map}, it is
     * considered as a composite type, and converted to the given {@code type}
     * by instanciating it and populating each propety with the result of
     * {@link #unmap(Object, Class)} applied to each matching key</li>
     * <li>Otherwise, the object is considered a simple type, and coerced to the
     * given {@code type}</li>
     * </ol>
     *
     * @param value
     * @param type
     * @return the SLM converted to an instance of the target type, or {@code null}, if {@code map} is {@code null}
     * @throws IllegalArgumentException
     *             if the object can not be converted into an instance of the
     *             requested type, because of unmatching keys or uncoersable
     *             properties
     */
    Object unmap(Object value, @NonNull Class<?> type);

    /**
     * Conversion rules are the following:
     *
     * <ol>
     * <li>If the object is a {@link java.util.List}, it returns a list with
     * each element recursively converted</li>
     * <li>If the object is a an instance of a class considered by this
     * {@link MapObjectMapper} as composite data type, is is converted into a
     * map, with each key recursively converted</li>
     * <li>Otherwise, the object is considered a simple data type and returned</li>
     * </ol>
     *
     * @param value
     * @return
     */
    Object map(Object value);

}
