/**
 * Copyright (c) 2009-2011 Zauber S.A. <http://www.zaubersoftware.com/>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ar.com.zauber.commons.mom;

import net.sf.staccatocommons.restrictions.check.NonNull;

import ar.com.zauber.commons.mom.converter.TypeConverter;
import ar.com.zauber.commons.mom.style.SetterStyle;

/**
 * MapObjectMapperBuilder
 *
 * @author flbulgarelli
 */
public interface MapObjectMapperBuilder {

    /**
     * Adds a new converter to the {@link MapObjectMapper} being constructed.
     *
     * @param converter
     * @return this
     */
    MapObjectMapperBuilder withConverter(@NonNull TypeConverter converter);

    /**
     * Configures the setter style of the {@link MapObjectMapper} being constructed
     * @param setterStyle
     * @return this
     */
    MapObjectMapperBuilder withSetterStyle(@NonNull SetterStyle setterStyle);

    /**
     * Configures the interceptor of the {@link MapObjectMapper} being constructed.
     *
     * @param interceptor
     * @return this
     */
    MapObjectMapperBuilder withInterceptor(@NonNull MapObjectMapperInterceptor interceptor);

    /**
     * Answers a built, new {@link MapObjectMapper}. This message may be sent multiples
     * times.
     *
     * @return a new {@link MapObjectMapper}
     */
    MapObjectMapper build();

}
