/**
 * Copyright (c) 2009-2011 Zauber S.A. <http://www.zaubersoftware.com/>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ar.com.zauber.commons.mom;

import ar.com.zauber.commons.mom.converter.ConvertUtilsTypeConverter;
import ar.com.zauber.commons.mom.converter.IdentityTypeConverter;
import ar.com.zauber.commons.mom.converter.impl.DateToXMLGregorianCalendarTypeConverter;
import ar.com.zauber.commons.mom.converter.impl.StringToEnumTypeConverter;
import ar.com.zauber.commons.mom.internal.DefaultMapObjectMapper;
import ar.com.zauber.commons.mom.internal.DefaultMapObjectMapperBuilder;
import ar.com.zauber.commons.mom.style.impl.BeanStyle;

import net.sf.staccatocommons.restrictions.check.NotEmpty;

import org.apache.commons.beanutils.ConvertUtilsBean;

/**
 * {@link MapObjectMappers} is a hub for creating common
 * {@link MapObjectMapper}s and {@link MapObjectMapperBuilder}s
 *
 * @author flbulgarelli
 */
public class MapObjectMappers {

    private static final ConvertUtilsTypeConverter beanUtilsConverter = new ConvertUtilsTypeConverter(
            new ConvertUtilsBean());

    /**
     * Answers a default {@link MapObjectMapper}, which has {@link BeanStyle} as
     * setterSyle, with no converters no interceptor
     *
     * @param packageName
     * @return a new basic {@link MapObjectMapper} for the given pacakgeName
     */
    public static MapObjectMapper basic(@NotEmpty String packageName) {
        return new DefaultMapObjectMapper(packageName, BeanStyle.STYLE, IdentityTypeConverter.converter());
    }

    public static MapObjectMapperBuilder withPackage(@NotEmpty String packageName) {
        return new DefaultMapObjectMapperBuilder(packageName);
    }

    public static MapObjectMapperBuilder defaultWithPackage(@NotEmpty String packageName) {
        return withPackage(packageName) //
                .withConverter(DateToXMLGregorianCalendarTypeConverter.converter()) //
                .withConverter(StringToEnumTypeConverter.converter()) //
                .withConverter(beanUtilsConverter);
    }

    public static MapObjectMapper default_(@NotEmpty String packageName) {
        return defaultWithPackage(packageName).build();
    }



}
