/**
 * Copyright (c) 2009-2011 Zauber S.A. <http://www.zaubersoftware.com/>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ar.com.zauber.commons.mom.converter;

import static net.sf.staccatocommons.lambda.Lambda.$;
import static net.sf.staccatocommons.lambda.Lambda.lambda;

import net.sf.staccatocommons.collections.stream.Streams;
import net.sf.staccatocommons.restrictions.check.NonNull;
import net.sf.staccatocommons.restrictions.processing.ForceRestrictions;

/**
 *
 * CompositeTypeConverter
 * @author flbulgarelli
 */
public class CompositeTypeConverter implements TypeConverter {

    private final Iterable<TypeConverter> typeConverters;

    public CompositeTypeConverter(@NonNull Iterable<TypeConverter> typeConverters) {
        this.typeConverters = typeConverters;
    }

    @ForceRestrictions
    public boolean canConvert(@NonNull Class<?> sourceType, @NonNull Class<?> destinationType) {
        return Streams
            .from(typeConverters)
            .any(lambda($(TypeConverter.class).canConvert(sourceType,destinationType)));
    }

    @ForceRestrictions
    public <A> A convert(@NonNull Object value, @NonNull Class<A> destinationType) {

        return Streams
            .from(typeConverters)
            .find(lambda($(TypeConverter.class).canConvert(value.getClass(),destinationType)))
            .convert(value, destinationType);
    }

}
