/**
 * Copyright (c) 2009-2011 Zauber S.A. <http://www.zaubersoftware.com/>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ar.com.zauber.commons.mom.converter;

import net.sf.staccatocommons.restrictions.check.NonNull;
import net.sf.staccatocommons.restrictions.processing.ForceRestrictions;

import org.apache.commons.beanutils.ConvertUtilsBean;

/**
 * {@link ConvertUtilsTypeConverter}s are a {@link TypeConverter}s that are based on
 * Apache Bean Utils conversion framework.
 *
 * @author flbulgarelli
 */
public class ConvertUtilsTypeConverter implements TypeConverter
{
    private ConvertUtilsBean convertUtils;

    public ConvertUtilsTypeConverter(@NonNull ConvertUtilsBean convertUtils)
    {
        this.convertUtils = convertUtils;
    }

    @ForceRestrictions
    public boolean canConvert(@NonNull Class<?> sourceType, @NonNull  Class<?> destinationType)
    {
        return convertUtils.lookup(sourceType, destinationType) != null;
    }

    @ForceRestrictions
    @SuppressWarnings("unchecked")
    public <A> A convert(Object value, @NonNull  Class<A> destinationType)
    {
        return (A) convertUtils.convert(value, destinationType);
    }

}
