/**
 * Copyright (c) 2009-2011 Zauber S.A. <http://www.zaubersoftware.com/>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ar.com.zauber.commons.mom.internal;

import ar.com.zauber.commons.mom.AbstractMapObjectMapper;
import ar.com.zauber.commons.mom.converter.TypeConverter;
import ar.com.zauber.commons.mom.style.SetterStyle;

import java.lang.reflect.InvocationTargetException;
import java.util.Map;
import java.util.Map.Entry;

import net.sf.staccatocommons.check.Ensure;
import net.sf.staccatocommons.lang.SoftException;
import net.sf.staccatocommons.restrictions.check.NonNull;
import net.sf.staccatocommons.restrictions.processing.ForceRestrictions;

/**
 * {@link MapObjectMapper}
 *
 * @author flbulgarelli
 */
@SuppressWarnings( { "unchecked" })
public class DefaultMapObjectMapper extends AbstractMapObjectMapper {

    private final MappingPackage mappingPackage;
    private final SetterStyle setterStyle;
    private TypeConverter typeConverter;

    public DefaultMapObjectMapper(final String packageName,
            final SetterStyle setterStyle, TypeConverter typeConverter) {
        this.mappingPackage = new MappingPackage(packageName);
        this.setterStyle = setterStyle;
        this.typeConverter = typeConverter;
    }

    @Override
    @ForceRestrictions
    public boolean isCompositeType(@NonNull final Class<?> clazz) {
        return mappingPackage.isMappeable(clazz);
    }

    @Override
    protected Map<String, Object> mapObjectImpl(Object value) {
        Ensure.that(this.isCompositeType(value.getClass()), "%s must be a composite type", value);
        return new AttributesMap(value, this);
    }

    @Override
    protected Object coerce(Object value, Class<?> type) {
        return typeConverter.convert(value, type);
    }

    @Override
    protected <A> A unmapMapImpl(Map<String, Object> map, Class<A> targetType) {
        try {
            Object bean = targetType.newInstance();
            unmapProperties(map, bean);
            return (A) bean;
        } catch (Exception e) {
            throw SoftException.soften(e);
        }
    }

    protected void unmapProperties(
            @NonNull Map<String, Object> source,
            @NonNull Object destination) throws IllegalAccessException,
            InvocationTargetException {
        for (Entry<String, Object> property : source.entrySet())
            if(!property.getKey().equals(JAVA_TYPE_KEY))
                unmapProperty(property.getValue(), destination, property.getKey());
    }

    protected void unmapProperty(Object propertyValue, Object destination,
            String propertyName) {
        Class targetType = StructureType.getStructureType(destination,
                propertyName);
        try {
            setterStyle.setValue(destination,
                    propertyName, this.unmap(propertyValue, targetType));
        } catch (Exception e) {
            throw SoftException.soften(e);
        }
    }
}
